@@ -47,6 +47,15 @@ class Match(NamedTuple):
         ), f"{self.position}=={other.position} and {self.clause}=={other.clause}"
         return self.priority < other.priority or self.length > other.length
 
+    def __repr__(self):
+        return (
+            f"{self.__class__.__name__}(length={self.length}, "
+            f"sub_matches=<{len(self.sub_matches)}>, "
+            f"position={self.position}, "
+            f"clause={self.clause!r}, "
+            f"priority={self.priority})"
+        )
+
 
 class MemoTable(Generic[D]):
     __slots__ = ("matches", "source")
@@ -59,7 +68,7 @@ class MemoTable(Generic[D]):
         try:
             return self.matches[item]
         except KeyError:
-            if isinstance(item.clause, Not):
+            if type(item.clause) is Not:
                 match = item.clause.match(self.source, item.position, self)
                 if match is not None:
                     self.matches[item] = match
@@ -221,16 +230,17 @@ class Literal(Terminal[D]):
     A terminal matching a fixed literal
     """
 
-    __slots__ = ("value",)
+    __slots__ = ("value", "_len")
     maybe_zero = False
 
     def __init__(self, value: D):
         assert value, "Literal may not be empty"
         self.value = value
+        self._len = len(value)
 
     def match(self, source: D, at: int, memo: MemoTable):
-        if source[at : at + len(self.value)] == self.value:
-            return Match(len(self.value), (), at, self)
+        if source[at : at + self._len] == self.value:
+            return Match(self._len, (), at, self)
         return None
 
     def __eq__(self, other):
@@ -274,7 +284,6 @@ class Sequence(Clause[D]):
     def __init__(self, *sub_clauses: Clause[D]):
         self.sub_clauses = sub_clauses
         self._maybe_zero = None
-        self._hash = None
 
     def match(self, source: D, at: int, memo: MemoTable):
         offset, matches = at, ()
@@ -317,7 +326,6 @@ class Choice(Clause[D]):
     def __init__(self, *sub_clauses: Clause[D]):
         self.sub_clauses = sub_clauses
         self._maybe_zero = None
-        self._hash = None
 
     def match(self, source: D, at: int, memo: MemoTable):
         for index, sub_clause in enumerate(self.sub_clauses):
@@ -365,7 +373,6 @@ class Repeat(Clause[D]):
 
     def __init__(self, sub_clause: Clause[D]):
         self._sub_clause = sub_clause
-        self._hash = None
 
     def match(self, source: D, at: int, memo: MemoTable):
         try:
@@ -420,7 +427,6 @@ class Not(Clause[D]):
 
     def __init__(self, sub_clause: Clause[D]):
         self._sub_clause = sub_clause
-        self._hash = None
 
     def match(self, source: D, at: int, memo: MemoTable):
         try:
@@ -444,6 +450,51 @@ class Not(Clause[D]):
         return f"!{nested_str(self._sub_clause)}"
 
 
+class And(Clause[D]):
+    """
+    The lookahead of a clause, matching if the sub_clause matches but not consuming it
+    """
+
+    __slots__ = ("_sub_clause", "_hash")
+
+    @property
+    def maybe_zero(self):
+        assert not self._sub_clause.maybe_zero, "lookahead of zero match"
+        return False
+
+    @property
+    def sub_clauses(self) -> Tuple[Clause[D]]:
+        return (self._sub_clause,)
+
+    @sub_clauses.setter
+    def sub_clauses(self, values: Tuple[Clause[D]]):
+        (self._sub_clause,) = values
+
+    def __init__(self, sub_clause: Clause[D]):
+        self._sub_clause = sub_clause
+
+    def match(self, source: D, at: int, memo: MemoTable):
+        try:
+            sub_match = memo[MemoKey(at, self._sub_clause)]
+        except KeyError:
+            return None
+        else:
+            return Match(0, (sub_match,), at, self)
+
+    def __eq__(self, other):
+        return isinstance(other, And) and self._sub_clause == other._sub_clause
+
+    @cache_hash
+    def __hash__(self):
+        return hash(self._sub_clause)
+
+    def __repr__(self):
+        return f"{self.__class__.__name__}({self._sub_clause!r})"
+
+    def __str__(self):
+        return f"&{nested_str(self._sub_clause)}"
+
+
 # Grammar Definitions
 # TODO: Add Actions/Rules/Transforms
 class UnboundReference(LookupError):
@@ -482,7 +533,6 @@ class Reference(Clause[D]):
     def __init__(self, target: str):
         self.target = target
         self._sub_clause = None
-        self._hash = None
         # TODO: Correct?
         self._maybe_zero: Optional[bool] = None
 
