@@ -10,8 +10,11 @@ from bootpeg.pika.front import (
     Choice,
     Repeat,
     Not,
+    And,
+    Reference,
     Range,
     Delimited,
+    CapturedParseFailure,
 )
 
 
@@ -25,6 +28,8 @@ clauses = [
     Repeat(Literal("x")),
     Repeat(Sequence(Literal("x"), Literal("y"), Literal("z"))),
     Not(Literal("a")),
+    And(Literal("a")),
+    Reference("some_rule"),
     Range("a", "b"),
     Delimited(Literal("'"), Literal("'")),
 ]
@@ -39,3 +44,16 @@ def test_roundtrip(clause):
         "parse_test"
     ]
     assert parsed_rule.sub_clauses[0] == clause
+
+
+commit_failures = (("top:\n | ~\n", {9}),)
+
+
+@pytest.mark.parametrize("source, positions", commit_failures)
+def test_commit(source, positions):
+    try:
+        bpeg.parse(source)
+    except CapturedParseFailure as cpf:
+        assert cpf.positions == positions
+    else:
+        assert not positions, "Expected parse failures, found none"
